@@ -50,10 +50,14 @@ class Symbol:
         self.extent_start = -1
         self.extent_end = -1
         self.link = None
-        self.skip = False
         self.project_name = None
         self.parent_name = None
 
+    def __repr__(self):
+        return "%s(unique_name=%s, filename=%s, project=%s)" % (
+            type(self).__name__, self.unique_name, self.filename,
+            self.project_name)
+
     @classmethod
     def get_plural_name(cls):
         """Default implementation of the vmethod
@@ -109,8 +113,11 @@ class Symbol:
             elif type(sym) == Symbol:
                 sym.comment = self.comment.params.get(sym.display_name)
 
-    def _make_name(self):
-        return self.display_name
+            if isinstance(sym, Symbol):
+                sym.update_children_comments()
+
+    def make_name(self):
+        return self.display_name or self.unique_name
 
     def get_extra_links(self):
         """
@@ -129,7 +136,7 @@ class Symbol:
         Banana banana
         """
         if self.link is None:
-            self.link = Link(self.unique_name, self._make_name(),
+            self.link = Link(self.unique_name, self.make_name(),
                              self.unique_name)
 
         self.link = link_resolver.upsert_link(self.link, overwrite_ref=True)
@@ -143,6 +150,7 @@ class QualifiedSymbol:
     """
     Banana banana
     """
+
     def __init__(self, type_tokens=None):
         self.input_tokens = type_tokens or []
         self.comment = None
@@ -202,6 +210,7 @@ class ReturnItemSymbol(QualifiedSymbol):
     """
     Banana banana
     """
+
     def __init__(self, comment=None, name=None, **kwargs):
         QualifiedSymbol.__init__(self, **kwargs)
         self.comment = comment
@@ -212,6 +221,7 @@ class ParameterSymbol(QualifiedSymbol):
     """
     Banana banana
     """
+
     def __init__(self, argname='', comment=None, **kwargs):
         QualifiedSymbol.__init__(self, **kwargs)
         # FIXME: gir specific
@@ -232,7 +242,7 @@ class FieldSymbol(Symbol):
         self.member_name = None
         Symbol.__init__(self, **kwargs)
 
-    def _make_name(self):
+    def make_name(self):
         return self.member_name
 
     def get_children_symbols(self):
