@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
-import os, sys, linecache, pkgconfig, glob, subprocess
+import os, sys, linecache, pkgconfig, glob, subprocess, shutil
 
 from hotdoc.extensions.c.clang import cindex
 from ctypes import *
@@ -29,12 +29,18 @@ from hotdoc.core.symbols import *
 from hotdoc.core.comment import comment_from_tag
 from hotdoc.core.links import Link
 
-from hotdoc.parsers.gtk_doc import GtkDocParser
+from hotdoc.parsers.gtk_doc import GtkDocParser, gather_links, search_online_links
+from hotdoc.extensions.gi.gi_extension import GIExtension
 from hotdoc.extensions.c.utils import CCommentExtractor
 
 from hotdoc.utils.loggable import (info as core_info, warn, Logger,
     debug as core_debug)
 
+
+if shutil.which('llvm-config') is None:
+    raise ImportError()
+
+
 def ast_node_is_function_pointer (ast_node):
     if ast_node.kind == cindex.TypeKind.POINTER and \
             ast_node.get_pointee().get_result().kind != \
@@ -98,7 +104,9 @@ class ClangScanner(object):
         self.__doc_db = doc_db
         self.__all_sources = []
 
-    def scan(self, filenames, options, incremental, full_scan,
+        self.__renamed_symbols = {}
+
+    def scan(self, filenames, options, full_scan,
              full_scan_patterns, fail_fast=False, all_sources=None):
         if all_sources is None:
             self.__all_sources = []
@@ -147,6 +155,9 @@ class ClangScanner(object):
                             header_guarded.add(fname)
                     self.__parse_file (fname, tu, full_scan)
 
+        for unique_name, target in self.__renamed_symbols.items():
+            self.__doc_db.rename_symbol(unique_name, target)
+
         if not full_scan:
             comment_parser = GtkDocParser(self.project)
             CCommentExtractor(self.__doc_db, comment_parser).parse_comments(
@@ -214,7 +225,7 @@ class ClangScanner(object):
                 if not str(node.location.file) in self.filenames:
                     continue
 
-            if node.spelling in self.symbols:
+            if node.spelling in self.symbols or node.spelling in self.__renamed_symbols:
                 continue
 
             sym = None
@@ -232,6 +243,7 @@ class ClangScanner(object):
 
             if sym is not None:
                 self.symbols[sym.unique_name] = sym
+
             self.__create_symbols(node.get_children(), tu)
 
     def __getFunctionDeclNode(self, node):
@@ -246,7 +258,7 @@ class ClangScanner(object):
         if node.kind != cindex.CursorKind.COMPOUND_STMT:
             return None
 
-        if node.semantic_parent.kind == cindex.CursorKind.FUNCTION_DECL:
+        if node.semantic_parent and node.semantic_parent.kind == cindex.CursorKind.FUNCTION_DECL:
             return node.semantic_parent
 
         return None
@@ -272,7 +284,7 @@ class ClangScanner(object):
 
             tokens.append (link)
             self.__apply_qualifiers(type_, tokens)
-        elif type_.kind == cindex.TypeKind.UNEXPOSED:
+        elif type_.kind in (cindex.TypeKind.UNEXPOSED, cindex.TypeKind.ELABORATED):
             d = type_.get_declaration()
             if d.spelling:
                 tokens.append(Link(None, d.displayname, d.displayname))
@@ -308,10 +320,9 @@ class ClangScanner(object):
         if not return_value:
             return_value = [ReturnItemSymbol(type_tokens=[])]
 
-        sym = self.__doc_db.get_or_create_symbol(CallbackSymbol, parameters=parameters,
+        return self.__doc_db.create_symbol(CallbackSymbol, parameters=parameters,
                 return_value=return_value, display_name=node.spelling,
                 filename=str(node.location.file), lineno=node.location.line)
-        return sym
 
     def __parse_public_fields (self, decl):
         tokens = decl.translation_unit.get_tokens(extent=decl.extent)
@@ -415,10 +426,12 @@ class ClangScanner(object):
         member_name = field.spelling
         if parent_name:
             member_name = parent_name + '.' + member_name
-        member = self.__doc_db.get_or_create_symbol(FieldSymbol, is_function_pointer=is_function_pointer,
+        member = self.__doc_db.create_symbol(FieldSymbol, is_function_pointer=is_function_pointer,
                 member_name=member_name, qtype=qtype, filename=str(field.location.file),
                 display_name=name, unique_name=name)
-        members.append (member)
+
+        if member:
+            members.append (member)
 
         return last_record
 
@@ -437,7 +450,7 @@ class ClangScanner(object):
 
         anonymous = not node.spelling
 
-        return self.__doc_db.get_or_create_symbol(StructSymbol, raw_text=raw_text,
+        return self.__doc_db.create_symbol(StructSymbol, raw_text=raw_text,
                 members=members, anonymous=anonymous,
                 display_name=spelling,
                 filename=str(node.location.file), lineno=node.location.line)
@@ -448,11 +461,13 @@ class ClangScanner(object):
         for member in node.get_children():
             member_value = member.enum_value
             # FIXME: this is pretty much a macro symbol ?
-            member = self.__doc_db.get_or_create_symbol(EnumMemberSymbol, display_name=member.spelling,
+            member = self.__doc_db.create_symbol(EnumMemberSymbol, display_name=member.spelling,
                     filename=str(member.location.file),
                     lineno=member.location.line)
-            member.enum_value = member_value
-            members.append (member)
+
+            if member:
+                member.enum_value = member_value
+                members.append (member)
 
         anonymous = not node.spelling
 
@@ -462,7 +477,7 @@ class ClangScanner(object):
             end)]
         raw_text = '\n'.join(original_lines)
 
-        return self.__doc_db.get_or_create_symbol(EnumSymbol, members=members,
+        return self.__doc_db.create_symbol(EnumSymbol, members=members,
                 anonymous=anonymous, raw_text=raw_text, display_name=spelling,
                 filename=str(node.location.file), lineno=node.location.line)
 
@@ -479,23 +494,33 @@ class ClangScanner(object):
 
         filename = str(node.location.file)
 
-        symb = self.__doc_db.get_or_create_symbol(AliasSymbol, aliased_type=aliased_type,
-                display_name=node.spelling, filename=filename,
-                lineno=node.location.line, extra=extra)
+        # We want to allow a few patterns for documentation aliases,
+        # for example typedef struct _Foo Foo should let us document
+        # _Foo with the Foo display name. As we don't want to parse
+        # each translation unit twice, we instead keep track of symbols
+        # to rename and rename them at the end of the parsing stage
+        if (len(type_tokens) == 2 and
+            type_tokens[0] in ('struct ', 'enum ') and
+            isinstance(type_tokens[1], Link) and
+            '_%s' % node.spelling == type_tokens[1].id_):
+            self.__renamed_symbols[node.spelling] = type_tokens[1].id_
+            return None
+        else:
+            return self.__doc_db.create_symbol(AliasSymbol, aliased_type=aliased_type,
+                    display_name=node.spelling, filename=filename,
+                    lineno=node.location.line, extra=extra)
 
     def __create_typedef_symbol (self, node):
         t = node.underlying_typedef_type
         decl = t.get_declaration()
         if ast_node_is_function_pointer (t):
-            sym = self.__create_callback_symbol (node)
+            return self.__create_callback_symbol (node)
         elif not decl.spelling and decl.kind == cindex.CursorKind.STRUCT_DECL: # typedef struct {} foo;
-            sym = self.__create_struct_symbol (decl, spelling=node.spelling)
+            return self.__create_struct_symbol (decl, spelling=node.spelling)
         elif not decl.spelling and decl.kind == cindex.CursorKind.ENUM_DECL: # typedef enum {} bar;
-            sym = self.__create_enum_symbol (decl, spelling=node.spelling)
+            return self.__create_enum_symbol (decl, spelling=node.spelling)
         else:
-            sym = self.__create_alias_symbol (node)
-
-        return sym
+            return self.__create_alias_symbol (node)
 
     def __create_function_symbol (self, node):
         parameters = []
@@ -509,14 +534,12 @@ class ClangScanner(object):
                     type_tokens=type_tokens)
             parameters.append (parameter)
 
-        sym = self.__doc_db.get_or_create_symbol(FunctionSymbol, parameters=parameters,
+        return self.__doc_db.create_symbol(FunctionSymbol, parameters=parameters,
                 return_value=return_value, display_name=node.spelling,
                 filename=str(node.location.file), lineno=node.location.line,
                 extent_start=node.extent.start.line,
                 extent_end=node.extent.end.line)
 
-        return sym
-
     def __create_exported_variable_symbol (self, node):
         l = linecache.getline (str(node.location.file), node.location.line)
         split = l.split()
@@ -531,10 +554,9 @@ class ClangScanner(object):
         type_tokens = self.make_c_style_type_name(node.type)
         type_qs = QualifiedSymbol(type_tokens=type_tokens)
 
-        sym = self.__doc_db.get_or_create_symbol(ExportedVariableSymbol, original_text=original_text,
+        return self.__doc_db.create_symbol(ExportedVariableSymbol, original_text=original_text,
                 display_name=node.spelling, filename=str(node.location.file),
                 lineno=node.location.line, type_qs=type_qs)
-        return sym
 
 
 def flags_from_config(config):
@@ -581,8 +603,7 @@ class CExtension(Extension):
 
         if not symbol:
             scanner = ClangScanner(self.app, self.project, self)
-            scanner.scan([include_path], self.flags,
-                         self.app.incremental, True, ['*.c', '*.h'])
+            scanner.scan([include_path], self.flags, True, ['*.c', '*.h'])
             symbol = self.app.database.get_symbol(symbol_name)
 
             if not symbol:
@@ -610,25 +631,36 @@ class CExtension(Extension):
 
         return None
 
+    def _get_smart_key(self, symbol):
+        return os.path.splitext(symbol.filename)[0]
+
+    def _get_comment_smart_key(self, comment):
+        return os.path.splitext(comment.filename)[0]
+
     def _get_smart_index_title(self):
         return 'C API Reference'
 
-    def get_or_create_symbol(self, *args, **kwargs):
+    def create_symbol(self, *args, **kwargs):
         kwargs['language'] = 'c'
-        return super(CExtension, self).get_or_create_symbol(*args, **kwargs)
+        return super(CExtension, self).create_symbol(*args, **kwargs)
 
     def setup(self):
         super(CExtension, self).setup()
-        stale, unlisted = self.get_stale_files(self.sources)
-        self.scanner.scan(stale, self.flags,
-                          self.app.incremental, False, ['*.h'],
+        gather_links ()
+        self.scanner.scan(self.sources, self.flags, False, ['*.h'],
                           all_sources=self.sources)
 
+    def format_page(self, page, link_resolver, output):
+        link_resolver.get_link_signal.connect(search_online_links)
+        super().format_page(page, link_resolver, output)
+        link_resolver.get_link_signal.disconnect(
+            search_online_links)
+
     @staticmethod
     def add_arguments (parser):
         group = parser.add_argument_group('C extension', DESCRIPTION)
         CExtension.add_index_argument(group)
-        CExtension.add_sources_argument(group)
+        CExtension.add_sources_argument(group, add_root_paths=True)
         CExtension.add_paths_argument(group, "include-directories",
                 help_="List extra include directories here")
         group.add_argument ("--pkg-config-packages", action="store", nargs="+",
