@@ -16,6 +16,7 @@
 # along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
 from hotdoc.core.symbols import *
+from hotdoc.extensions.devhelp.devhelp_extension import TYPE_MAP
 
 class GIClassSymbol(ClassSymbol):
     def __init__(self, **kwargs):
@@ -24,3 +25,21 @@ class GIClassSymbol(ClassSymbol):
 
     def get_children_symbols(self):
         return [self.class_struct_symbol] + super().get_children_symbols()
+
+
+class GIStructSymbol(ClassSymbol):
+    """Boxed types are pretty much handled like classes with a possible
+       constructors, methods, etc... we render them as such.
+    """
+    def __init__(self, **kwargs):
+        self.class_struct_symbol = None
+        ClassSymbol.__init__(self, **kwargs)
+
+
+class FundamentalSymbol(Symbol):
+    __tablename__ = 'fundamentals'
+
+    def get_type_name(self):
+        return 'Fundamental'
+
+TYPE_MAP[GIClassSymbol] = 'class'
