@@ -27,20 +27,31 @@ import re
 import cgi
 from collections import OrderedDict
 from itertools import zip_longest
+from xdg import XDG_DATA_HOME, XDG_DATA_DIRS
+from lxml import etree
 
+import yaml
+from yaml.constructor import ConstructorError
 
 from hotdoc.core.comment import (Comment, Annotation, Tag,
                                  comment_from_tag)
 from hotdoc.core.exceptions import HotdocSourceException
+from hotdoc.core.links import Link
 from hotdoc.utils.configurable import Configurable
 from hotdoc.parsers import cmark
-from hotdoc.utils.loggable import Logger, warn
-
+from hotdoc.utils.loggable import Logger, warn, info
 
+Logger.register_warning_code('gtk-doc', HotdocSourceException)
 Logger.register_warning_code('gtk-doc-bad-link', HotdocSourceException)
 Logger.register_warning_code('gtk-doc-bad-syntax', HotdocSourceException)
 
 
+GTKDOC_HREFS = {}
+
+
+GATHERED_GTKDOC_LINKS = False
+
+
 # http://stackoverflow.com/questions/434287/what-is-the-most-pythonic-way-to-iterate-over-a-list-in-chunks
 def _grouper(iterable, n_args, fillvalue=None):
     """
@@ -83,33 +94,21 @@ class GtkDocParser:
 
     def __parse_title(self, source_filename, raw_title):
         if raw_title.startswith('SECTION'):
-            rel_filename = raw_title.split('SECTION:')[1].strip()
-            section_name = os.path.abspath(os.path.join(
-                os.path.dirname(source_filename), rel_filename))
-
-            if not os.path.exists(section_name):
-                fname, ext = os.path.splitext(source_filename)
-                if self.__section_file_matching and os.path.exists(
-                        source_filename) and ext == ".c" and os.path.exists(
-                            fname + '.h'):
-                    section_name = source_filename[:-2] + '.h'
-                else:
-                    section_name = rel_filename
-
-            return section_name, []
+            section_name = raw_title.split('SECTION:')[1].strip()
+            return section_name, [], True
 
         split = raw_title.split(': ', 1)
 
-        if not split[0].rstrip().endswith(':'):
+        title = split[0].rstrip(':')
+        if not len(split) > 1 and not title.endswith(':') and " " in title:
             raise HotdocSourceException(
                 message='Unexpected title in gtk-doc comment')
 
-        title = split[0].rstrip(':')
         annotations = []
 
         if len(split) > 1:
             annotations = self.__parse_annotations(split[1])
-        return title, annotations
+        return title, annotations, False
 
     def __parse_key_value_annotation(self, name, string):
         kvs = self.kv_regex.findall(string)
@@ -169,16 +168,17 @@ class GtkDocParser:
 
     def __parse_parameter(self, name, desc):
         name = name.strip()[1:-1].strip()
+        raw_comment = '%s:%s' % (name, desc)
         desc = desc.strip()
         desc, annotations = self.__extract_annotations(desc)
         annotations = {annotation.name: annotation for annotation in
                        annotations}
         return Comment(name=name, annotations=annotations,
-                       description=desc)
+                       meta={'description': desc}, raw_comment=raw_comment)
 
     def __parse_title_and_parameters(self, filename, title_and_params):
         tps = re.split(r'(\n[ \t]*@[\S]+[ \t]*:)', title_and_params)
-        title, annotations = self.__parse_title(filename, tps[0])
+        title, annotations, is_section = self.__parse_title(filename, tps[0])
         parameters = []
         for name, desc in _grouper(tps[1:], 2):
             n_chars = len(name.strip()) + len(desc)
@@ -186,7 +186,7 @@ class GtkDocParser:
             param.line_offset = len(desc.split('\n'))
             parameters.append(param)
             param.initial_col_offset = n_chars - len(param.description)
-        return title, parameters, annotations
+        return title, parameters, annotations, is_section
 
     # pylint: disable=no-self-use
     def __parse_since_tag(self, name, desc):
@@ -239,10 +239,10 @@ class GtkDocParser:
 
         validator = self.project.tag_validators.get(name)
         if not validator:
-            print("FIXME no tag validator")
+            warn('gtk-doc', "FIXME no tag validator")
             return None
         if not validator.validate(desc):
-            print("invalid value for tag %s : %s" % name, desc)
+            warn('gtk-doc', "invalid value for tag %s : %s" % (name, desc))
             return None
         return Tag(name=name, description=desc, value=desc)
 
@@ -276,9 +276,38 @@ class GtkDocParser:
         return re.match(r'(/\*\*([^*]|[\r\n]|(\*+([^*/]|[\r\n])))*\*+/)$',
                         comment) is not None
 
+    def __parse_yaml_comment(self, comment, filename):
+        res = {}
+        try:
+            blocks = yaml.load_all(comment.raw_comment, Loader=yaml.SafeLoader)
+            for block in blocks:
+                if block:
+                    res.update(block)
+        except (ConstructorError, yaml.parser.ParserError) as exception:
+            warn('invalid-page-metadata',
+                 '%s: Invalid metadata: \n%s' % (filename, str(exception)))
+        return res
+
+    def __extract_titles_params_and_description(self, comment):
+        titleandparams_description = re.split(r'\n[\s]*\n', comment, maxsplit=1)
+        title_and_params = titleandparams_description[0]
+
+        title_and_params_lines = title_and_params.split('\n')
+        if len(title_and_params_lines) > 1 and not title_and_params_lines[1].strip().startswith('@'):
+            return title_and_params_lines[0].rstrip(':'), '\n'.join([l.strip() for l in title_and_params_lines[1:]])
+
+        if len(titleandparams_description) > 1:
+            description = titleandparams_description[1]
+        else:
+            description = None
+
+        return title_and_params, description
+
     # pylint: disable=too-many-arguments
     # pylint: disable=too-many-locals
     # pylint: disable=unused-argument
+    # pylint: disable=too-many-branches
+    # pylint: disable=too-many-statements
     def parse_comment(self, comment, filename, lineno, endlineno,
                       include_paths=None, stripped=False):
         """
@@ -300,11 +329,10 @@ class GtkDocParser:
                 column_offset = 0
             comment, title_offset = self.__strip_comment(comment)
 
-        split = re.split(r'\n[\W]*\n', comment, maxsplit=1)
-
+        title_and_params, description = self.__extract_titles_params_and_description(comment)
         try:
-            block_name, parameters, annotations = \
-                self.__parse_title_and_parameters(filename, split[0])
+            block_name, parameters, annotations, is_section = \
+                self.__parse_title_and_parameters(filename, title_and_params)
         except HotdocSourceException as _:
             warn('gtk-doc-bad-syntax',
                  message=_.message,
@@ -325,22 +353,25 @@ class GtkDocParser:
             return None
 
         description_offset = 0
-        description = ""
+        meta = {}
         tags = []
-        if len(split) > 1:
+        if description is not None:
             n_lines = len(comment.split('\n'))
             description_offset = (title_offset + n_lines -
-                                  len(split[1].split('\n')))
-            description, tags = self.__parse_description_and_tags(split[1])
+                                  len(description.split('\n')))
+            meta['description'], tags = self.__parse_description_and_tags(description)
 
-        title = None
-        short_description = None
         actual_parameters = OrderedDict({})
         for param in parameters:
-            if param.name.lower() == 'short_description':
-                short_description = param
-            elif param.name.lower() == 'title':
-                title = param
+            if is_section:
+                cleaned_up_name = param.name.lower().replace('_', '-')
+                if cleaned_up_name in ['symbols', 'private-symbols', 'auto-sort', 'sources']:
+                    meta.update(self.__parse_yaml_comment(param, filename))
+                    if cleaned_up_name == 'sources':
+                        sources_paths = [os.path.abspath(os.path.join(os.path.dirname(filename), path)) for path in meta[cleaned_up_name]]
+                        meta[cleaned_up_name] = sources_paths
+                else:
+                    meta[param.name] = param.description
             else:
                 actual_parameters[param.name] = param
 
@@ -351,9 +382,8 @@ class GtkDocParser:
         block = Comment(name=block_name, filename=filename, lineno=lineno,
                         endlineno=endlineno,
                         annotations=annotations, params=actual_parameters,
-                        description=description,
-                        short_description=short_description,
-                        title=title, tags=tags, raw_comment=raw_comment)
+                        tags=tags, raw_comment=raw_comment,
+                        meta=meta, toplevel=is_section)
         block.line_offset = description_offset
         block.col_offset = column_offset
 
@@ -371,7 +401,7 @@ class GtkDocStringFormatter(Configurable):
         self.gdbus_codegen_sources = []
 
     # pylint: disable=no-self-use
-    def comment_to_ast(self, comment, link_resolver):
+    def comment_to_ast(self, comment, link_resolver, include_resolver):
         """
         Given a gtk-doc comment string, returns an opaque PyCapsule
         containing the document root.
@@ -408,11 +438,16 @@ class GtkDocStringFormatter(Configurable):
         if self.escape_html:
             # pylint: disable=deprecated-method
             text = cgi.escape(text)
-        ast, diagnostics = cmark.gtkdoc_to_ast(text, link_resolver)
+
+        ast, diagnostics = cmark.gtkdoc_to_ast(text, link_resolver, include_resolver, comment.filename)
 
         for diag in diagnostics:
             if (comment.filename and comment.filename not in
                     self.gdbus_codegen_sources):
+                # FIXME: update offset calculation code for includes
+                if diag.filename != comment.filename:
+                    diag.lineno = -1
+
                 column = diag.column + comment.col_offset
                 if diag.lineno == 0:
                     column += comment.initial_col_offset
@@ -429,13 +464,13 @@ class GtkDocStringFormatter(Configurable):
                     column += 1
 
                 lineno = -1
-                if comment.lineno != -1:
+                if comment.lineno != -1 and diag.lineno != -1:
                     lineno = (comment.lineno - 1 + comment.line_offset +
                               diag.lineno)
                 warn(
                     diag.code,
                     message=diag.message,
-                    filename=comment.filename,
+                    filename=diag.filename,
                     lineno=lineno,
                     column=column)
 
@@ -455,26 +490,26 @@ class GtkDocStringFormatter(Configurable):
         out, _ = cmark.ast_to_html(ast, link_resolver)
         return out
 
-    def translate_comment(self, comment, link_resolver):
+    def translate_comment(self, comment, link_resolver, include_resolver):
         """
         Given a gtk-doc comment string, returns the comment translated
         to the desired format.
         """
         out = u''
 
-        self.translate_tags(comment, link_resolver)
-        ast = self.comment_to_ast(comment, link_resolver)
+        self.translate_tags(comment, link_resolver, include_resolver)
+        ast = self.comment_to_ast(comment, link_resolver, include_resolver)
         out += self.ast_to_html(ast, link_resolver)
         return out
 
-    def translate_tags(self, comment, link_resolver):
+    def translate_tags(self, comment, link_resolver, include_resolver):
         """Banana banana
         """
         for tname in ('deprecated',):
             tag = comment.tags.get(tname)
             if tag is not None and tag.description:
                 comment = comment_from_tag(tag)
-                ast = self.comment_to_ast(comment, link_resolver)
+                ast = self.comment_to_ast(comment, link_resolver, include_resolver)
                 tag.description = self.ast_to_html(ast, link_resolver) or ''
 
     @staticmethod
@@ -501,3 +536,122 @@ class GtkDocStringFormatter(Configurable):
         self.remove_xml_tags = config.get('gtk_doc_remove_xml')
         self.escape_html = config.get('gtk_doc_escape_html')
         self.gdbus_codegen_sources = config.get_paths('gdbus_codegen_sources')
+
+
+def parse_devhelp_index(dir_):
+    path = os.path.join(dir_, os.path.basename(dir_) + '.devhelp2')
+    if not os.path.exists(path):
+        return False
+
+    try:
+        dh_root = etree.parse(path).getroot()
+    except etree.Error:
+        # No need to look for a sgml file
+        return True
+
+    online = dh_root.attrib.get('online')
+    name = dh_root.attrib.get('name')
+    author = dh_root.attrib.get('author')
+    language = dh_root.attrib.get('language')
+
+    if not online:
+        if not name:
+            return False
+        online = 'https://developer.gnome.org/%s/unstable/' % name
+
+    keywords = dh_root.findall('.//{http://www.devhelp.net/book}keyword')
+    for kw in keywords:
+        name = kw.attrib["name"]
+        type_ = kw.attrib['type']
+        link = kw.attrib['link']
+
+        if type_ in ['macro', 'function']:
+            name = name.rstrip(u' ()')
+        elif type_ in ['struct', 'enum', 'union']:
+            split = name.split(' ', 1)
+            if len(split) == 2:
+                name = split[1]
+            else:
+                name = split[0]
+        elif type_ in ['signal', 'property']:
+            # Heuristic to determine that the naming follows the gtk-doc "logic"
+            if '#' in link and (language.lower() == 'c' or author == 'hotdoc'):
+                anchor = link.split('#', 1)[1]
+                if author == 'hotdoc':
+                    name = anchor
+                else:
+                    split = anchor.split('-', 1)
+                    if type_ == 'signal':
+                        name = '%s::%s' % (split[0], split[1].lstrip('-'))
+                    else:
+                        name = '%s:%s' % (split[0], split[1].lstrip('-'))
+        elif type_ in ['vfunc']:
+            if '#' in link and (language.lower() == 'c' or author == 'hotdoc'):
+                anchor = link.split('#', 1)[1]
+                if author == 'hotdoc':
+                    name = anchor
+                    GTKDOC_HREFS[name.replace('::', '.')] = online + link
+
+        GTKDOC_HREFS[name] = online + link
+
+    return True
+
+
+def parse_sgml_index(dir_):
+    remote_prefix = ""
+    n_links = 0
+    path = os.path.join(dir_, "index.sgml")
+    with open(path, 'r') as f:
+        for l in f:
+            if l.startswith("<ONLINE"):
+                remote_prefix = l.split('"')[1]
+            elif not remote_prefix:
+                break
+            elif l.startswith("<ANCHOR"):
+                split_line = l.split('"')
+                filename = split_line[3].split('/', 1)[-1]
+                title = split_line[1].replace('-', '_')
+
+                if title.endswith(":CAPS"):
+                    title = title [:-5]
+                if remote_prefix:
+                    href = '%s/%s' % (remote_prefix, filename)
+                else:
+                    href = filename
+
+                GTKDOC_HREFS[title] = href
+                n_links += 1
+
+
+def gather_links():
+    global GATHERED_GTKDOC_LINKS
+
+    if GATHERED_GTKDOC_LINKS:
+        return
+
+    GATHERED_GTKDOC_LINKS = True
+
+    # XDG_DATA_DIRS is preference-ordered, we reverse so that preferred
+    # links override less-preferred ones
+    for datadir in reversed([XDG_DATA_HOME] + XDG_DATA_DIRS):
+        for path in (os.path.join(datadir, 'devhelp', 'books'), os.path.join(datadir, 'gtk-doc', 'html')):
+            if not os.path.exists(path):
+                info("no gtk doc to gather links from in %s" % path)
+                continue
+
+            for node in os.listdir(path):
+                dir_ = os.path.join(path, node)
+                if os.path.isdir(dir_):
+                    if not parse_devhelp_index(dir_):
+                        try:
+                            parse_sgml_index(dir_)
+                        except IOError:
+                            pass
+
+
+def search_online_links(resolver, name):
+    href = GTKDOC_HREFS.get(name)
+    if href:
+        return Link(href, name, name)
+    return None
+
