@@ -40,7 +40,7 @@ class TestParser(unittest.TestCase):
         self.link_resolver.add_link(Link("here.com", "foo", "foo"))
 
     def assertOutputs(self, inp, expected):
-        ast, _ = cmark.gtkdoc_to_ast(inp, self.link_resolver)
+        ast, _ = cmark.gtkdoc_to_ast(inp, self.link_resolver, None, None)
         out = cmark.ast_to_html(ast, self.link_resolver)[0]
         self.assertEqual(out, expected)
 
@@ -55,7 +55,7 @@ class TestParser(unittest.TestCase):
     def test_input_none(self):
         inp = None
         with self.assertRaises(TypeError):
-            ast, _ = cmark.gtkdoc_to_ast(inp, self.link_resolver)
+            ast, _ = cmark.gtkdoc_to_ast(inp, self.link_resolver, None, None)
             self.assertEqual(ast, None)
 
     def test_resolver_none(self):
@@ -71,6 +71,10 @@ BASIC_GTKDOC_COMMENT = '''/**
  * This is just a function.
  */'''
 
+SKIP_GTKDOC_COMMENT = '''/**
+ * test_greeter_greet: (skip)
+ */'''
+
 LINENOS_GTKDOC_COMMENT = '''/**
  *
  *
@@ -91,6 +95,20 @@ SECTION_GTKDOC_COMMENT = '''/**
  * @title: This page is not comming from a file.
  */'''
 
+SECTION_GTKDOC_WITH_SYMBOLS = '''/**
+ * SECTION: notafile
+ * @title: This page is not comming from a file.
+ * @auto-sort: true
+ * @symbols:
+ * - symbol_a
+ * - symbol_b
+ */'''
+
+SECTION_WITH_NO_NEWLINE_NOR_METAS = '''/**
+ * SECTION: this:
+ * "this" is a section.
+ */'''
+
 NOT_A_GTKDOC_COMMENT = '''/**
  * $param rd Pointer to vbi3_raw_decoder object allocated with
  *   vbi3_raw_decoder_new().
@@ -128,6 +146,17 @@ class TestGtkDocParser(unittest.TestCase):
         param = comment.params['greeter']
         self.assertEqual(param.description, 'a random greeter')
 
+    def test_skip_gtkdoc_comment(self):
+        raw = SKIP_GTKDOC_COMMENT
+        lineno = 10
+        end_lineno = len(raw.split('\n')) + 10 - 1
+        comment = self.parser.parse_comment(
+            raw,
+            None,
+            lineno,
+            end_lineno)
+        self.assertIn('skip', comment.annotations)
+
     def test_not_a_gtkdoc_comment(self):
         raw = NOT_A_GTKDOC_COMMENT
         lineno = 10
@@ -149,22 +178,30 @@ class TestGtkDocParser(unittest.TestCase):
             lineno,
             end_lineno)
         self.assertEqual(comment.name, "notafile")
+        self.assertEqual(comment.toplevel, True)
 
-    def test_page_name_filename(self):
-        raw_lines = SECTION_GTKDOC_COMMENT.split("\n")
-        tmpfile = tempfile.NamedTemporaryFile()
+    def test_section_with_no_newline_nor_metas(self):
+        raw_lines = SECTION_WITH_NO_NEWLINE_NOR_METAS.split("\n")
 
-        raw_lines[1] = " * SECTION: %s" % (
-            os.path.relpath(tmpfile.name, '.'))
-        raw = '\n'.join(raw_lines)
-        lineno = 10
-        end_lineno = len(raw.split('\n')) + 10 - 1
         comment = self.parser.parse_comment(
-            raw,
+            SECTION_WITH_NO_NEWLINE_NOR_METAS,
             'some-file.c',
-            lineno,
-            end_lineno)
-        self.assertEqual(comment.name, tmpfile.name)
+            0,
+            len(raw_lines) - 1)
+
+        self.assertEqual(comment.name, "this")
+        self.assertEqual(comment.description, '"this" is a section.')
+
+    def test_symbols_list(self):
+        raw_lines = SECTION_GTKDOC_WITH_SYMBOLS.split("\n")
+
+        comment = self.parser.parse_comment(
+            SECTION_GTKDOC_WITH_SYMBOLS,
+            'some-file.c',
+            0,
+            len(raw_lines) - 1)
+        self.assertEqual(comment.meta['auto-sort'], True)
+        self.assertEqual(comment.meta['symbols'], ['symbol_a', 'symbol_b'])
 
     def test_linenos(self):
         raw = LINENOS_GTKDOC_COMMENT
@@ -199,7 +236,7 @@ class TestGtkDocExtension(unittest.TestCase):
                                          "org.dbus.func"))
 
     def assertOutputs(self, inp, expected):
-        ast, diagnostics = cmark.gtkdoc_to_ast(inp, self.link_resolver)
+        ast, diagnostics = cmark.gtkdoc_to_ast(inp, self.link_resolver, None, None)
         out = cmark.ast_to_html(ast, self.link_resolver)[0]
         self.assertEqual(out, expected)
         return ast, diagnostics
@@ -363,7 +400,7 @@ class TestGtkDocExtension(unittest.TestCase):
         inp = (u'a #wrong_link\n\n'
                'and #another_wrong_link\n'
                'and then #yet_another_wrong_link')
-        _, diagnostics = cmark.gtkdoc_to_ast(inp, self.link_resolver)
+        _, diagnostics = cmark.gtkdoc_to_ast(inp, self.link_resolver, None, None)
         self.assertEqual(len(diagnostics), 3)
         diag = diagnostics[0]
         self.assertEqual(diag.lineno, 0)
@@ -398,7 +435,7 @@ class TestIncludeExtension(unittest.TestCase):
         self.include_resolver = MockIncludeResolver()
 
     def assertOutputs(self, inp, expected):
-        ast = cmark.hotdoc_to_ast(inp, self.include_resolver)
+        ast = cmark.hotdoc_to_ast(inp, self.include_resolver, None)
         out = cmark.ast_to_html(ast, self.link_resolver)[0]
         self.assertEqual(out, expected)
         return out, ast
@@ -460,7 +497,7 @@ class TestTableExtension(unittest.TestCase):
         self.include_resolver = MockIncludeResolver()
 
     def render(self, inp):
-        ast = cmark.hotdoc_to_ast(inp, self.include_resolver)
+        ast = cmark.hotdoc_to_ast(inp, self.include_resolver, None)
         out = cmark.ast_to_html(ast, self.link_resolver)[0]
         return ast, out
 
@@ -527,7 +564,7 @@ class TestTableExtension(unittest.TestCase):
 
 class TestAutoRefExtension(unittest.TestCase):
     def render(self, inp):
-        ast = cmark.hotdoc_to_ast(inp, None)
+        ast = cmark.hotdoc_to_ast(inp, None, None)
         out = cmark.ast_to_html(ast, None)[0]
         return ast, out
 
