@@ -21,6 +21,7 @@ Toolbox
 """
 
 import collections
+# pylint: disable=no-name-in-module
 from collections import OrderedDict, Callable
 import os
 import shutil
@@ -28,6 +29,7 @@ import math
 import sys
 import re
 import pathlib
+import traceback
 
 from urllib.request import urlretrieve
 
@@ -36,11 +38,17 @@ from toposort import toposort_flatten
 
 from hotdoc.core.exceptions import HotdocSourceException
 from hotdoc.utils.setup_utils import symlink
-from hotdoc.utils.loggable import error
+from hotdoc.utils.loggable import info, debug, error
 
 WIN32 = (sys.platform == 'win32')
 
 
+if os.name == 'nt':
+    DATADIR = os.path.join(os.path.dirname(__file__), '..', 'share')
+else:
+    DATADIR = "/usr/share"
+
+
 def splitall(path):
     """
     Splits path in its components:
@@ -98,6 +106,21 @@ def all_subclasses(cls):
                                    for g in all_subclasses(s)]
 
 
+def flatten_list(list_):
+    """
+    Banana banana
+    """
+    res = []
+
+    for elem in list_:
+        if isinstance(elem, list):
+            res.extend(flatten_list(elem))
+        else:
+            res.append(elem)
+
+    return res
+
+
 def get_mtime(filename):
     """
     Banana banana
@@ -127,7 +150,8 @@ def __get_extra_extension_classes(paths):
             classes = activation_function()
         # pylint: disable=broad-except
         except Exception as exc:
-            print("Failed to load %s %s" % (entry_point.module_name, exc))
+            info("Failed to load %s %s" % (entry_point.module_name, exc))
+            debug(traceback.format_exc())
             continue
 
         for klass in classes:
@@ -137,6 +161,7 @@ def __get_extra_extension_classes(paths):
 
 
 # pylint: disable=too-many-locals
+# pylint: disable=too-many-branches
 def get_extension_classes(sort, extra_extension_paths=None):
     """
     Banana banana
@@ -152,8 +177,9 @@ def get_extension_classes(sort, extra_extension_paths=None):
             activation_function = entry_point.load()
             classes = activation_function()
         # pylint: disable=broad-except
-        except Exception as _:
-            print("Failed to load %s" % entry_point.module_name, _)
+        except Exception as exc:
+            info("Failed to load %s" % entry_point.module_name, exc)
+            debug(traceback.format_exc())
             continue
 
         for klass in classes:
@@ -211,6 +237,9 @@ class OrderedSet(collections.MutableSet):
 
     # pylint: disable=arguments-differ
     def add(self, key):
+        """
+        Banana banana
+        """
         if key not in self.map:
             end = self.end
             curr = end[1]
@@ -236,6 +265,9 @@ class OrderedSet(collections.MutableSet):
 
     # pylint: disable=arguments-differ
     def discard(self, key):
+        """
+        Banana banana
+        """
         if key in self.map:
             key, prev, nxt = self.map.pop(key)
             prev[2] = nxt
@@ -257,6 +289,9 @@ class OrderedSet(collections.MutableSet):
 
     # pylint: disable=arguments-differ
     def pop(self, last=True):
+        """
+        Banana banana
+        """
         if not self:
             raise KeyError('set is empty')
         key = self.end[1][0] if last else self.end[2][0]
